/**
 * AngularJS controller capable of retrieving JSON/REST content
 * from our REST back-end 
 */

var config = {
	headers:  {
		'Accept': 'application/json'
	}
};

/**
 * Checks for typical errors and renders them a bit more pretty
 * 
 * @param error
 */
function renderError(error) {
	if(error.statusText && error.status) {
		// HTTP error
		return 'Server Said: ' + error.status + ': ' + error.statusText + ' when requesting ' + error.config.url;
	}
	return error;
}

var angularShopApp = angular.module('angularShopApp', []);

angularShopApp.controller('ItemController', ['$scope','$http', function($scope, $http) {
	// customer name
	$scope.customerName = 'Thomas Testhuster';
	
	// holds the items retrieved from REST service
	$scope.items = [];
	
	// items in our local shopping cart
	$scope.cart = [];
	
	// item in the edit pane
	$scope.editItem = {  };
	$scope.editItem.id = 0;
	$scope.editItem.name = '';
	$scope.editItem.price = 0.0;
	$scope.editItem.stock = 0;
	
    // function retrieve items from the back-end
    $scope.loadItems = function() {
    	$http.get('http://localhost:8080/shop/rest/items', config).then(function (response) {
    		$scope.items = response.data;
    		console.log('Received response');
    		console.log($scope.items);
    	}, function (error) {
    		$scope.errorMessage = renderError(error);
    		$scope.serverResponse = error.data;
    	});
    };
    
    // function to add an item to cart
    $scope.addToCart = function(item) {
    	var found = undefined;
    	for(var i=0;i<$scope.cart.length;i++) {
    		if($scope.cart[i].item.id == item.id) {
    			found = $scope.cart[i];
    		}
    	}
    	if(found) {
    		found.quantity += 1;
    	}else{
    		var order = {
    			item: item,
    			quantity: 1
    		};
    		$scope.cart.push(order);
    	}
    	console.log($scope.cart);
    };
    
    // order all items in the cart
    $scope.placeOrder = function() {
    	var cartItems = [];
    	for(var i=0;i<$scope.cart.length;i++) {
    		var orderItem = {
    			item: $scope.cart[i].item.id,
    			quantity: $scope.cart[i].quantity,
    			posPrice: 0.0
    		};
    		cartItems.push(orderItem);
    	}
    	var order = {
    		customerName: $scope.customerName,
    		items: cartItems
    	};
    	$http.post('http://localhost:8080/shop/rest/orders', order, config).then(function (response) {
    		$scope.emptyCart();
    		$scope.loadItems();
    		$scope.infoMessage = 'Order sucessfully placed order number: ' + response.data.id;
    	}, function (error) {
    		$scope.errorMessage = renderError(error);
    		$scope.serverResponse = error.data;
    	});
    };
    
    // delete all items from the cart
    $scope.emptyCart = function() {
    	$scope.cart = [];
    };
    
    // POST item to create NEW item
    $scope.postItem = function() {
    	$http.post('http://localhost:8080/shop/rest/items', $scope.editItem, config).then(function (response) {
    		$scope.loadItems();
    		$scope.infoMessage = 'Sucessfully created a new item: ' + response.data.id;
    		$scope.editItem = response.data;
    	}, function (error) {
    		$scope.errorMessage = renderError(error);
    		$scope.serverResponse = error.data;
    	});
    };
    
    // PUT item to UPDATE item
    $scope.putItem = function() {
    	$http.put('http://localhost:8080/shop/rest/items/' + $scope.editItem.id, $scope.editItem, config).then(function (response) {
    		$scope.loadItems();
    		$scope.infoMessage = 'Sucessfully updated the item: ' + response.data.id;
    		$scope.editItem = response.data;
    	}, function (error) {
    		$scope.errorMessage = renderError(error);
    		$scope.serverResponse = error.data;
    	});
    };
    
    // GET item to retrieve via REST
    $scope.getItem = function() {
    	$http.get('http://localhost:8080/shop/rest/items/' + $scope.editItem.id, config).then(function (response) {
    		$scope.editItem = response.data;
    	}, function (error) {
    		$scope.errorMessage = renderError(error);
    		$scope.serverResponse = error.data;
    	});
    };
    
    // DELETE an item
    $scope.deleteItem = function() {
    	$http.delete('http://localhost:8080/shop/rest/items/' + $scope.editItem.id, config).then(function (response) {
    		$scope.editItem = {};
    		$scope.editItem.id = 0;
    		$scope.editItem.name = '';
    		$scope.editItem.price = 0;
    		$scope.editItem.stock = 0;
    		$scope.loadItems();
    	}, function (error) {
    		$scope.errorMessage = renderError(error);
    		$scope.serverResponse = error.data;
    	});
    };
    
    // on controller start-up retrieve items from back-end
    $scope.loadItems();
}]);