package at.gv.landooe.shop.dao;

import java.io.File;
import java.util.concurrent.ConcurrentNavigableMap;

import org.apache.log4j.Logger;
import org.mapdb.DB;
import org.mapdb.DBMaker;

import at.gv.landooe.shop.Settings;
import at.gv.landooe.shop.elems.Item;

/**
 * Singleton access to a disk bound MapDB database in the users
 * $TMP directory 
 * 
 * @author matthias
 */
public class MapDBDatabaseConnection {
	
	static Logger logger = Logger.getLogger(MapDBDatabaseConnection.class);

	/** Manged singleton */
	private static DB databaseSingleton = null;
	
	public static synchronized DB getDb() {
		if(MapDBDatabaseConnection.databaseSingleton == null) {
			if(Settings.USE_FILE_DB) {
				String dbFileName = buildDatabaseTempFileName();
				MapDBDatabaseConnection.databaseSingleton = DBMaker
						.newFileDB(new File(dbFileName)).closeOnJvmShutdown()
						.encryptionEnable("password").make();
				logger.info("Created new database connection to file: " + dbFileName);
			}else{
				MapDBDatabaseConnection.databaseSingleton = DBMaker.newMemoryDB().make();
				logger.info("Created a volatile in-memory database");
			}
			ConcurrentNavigableMap<Integer, Item> treeMap = MapDBDatabaseConnection.databaseSingleton.getTreeMap(ShopDAO.ITEMS);
			Item item = treeMap.get(1);
			if(item == null) {
				logger.info("Database connection does not contain an item with id == 1 > populating DB with testdata");
				ShopDAO.runPopulateScenario(MapDBDatabaseConnection.databaseSingleton);
			}
		}
		return MapDBDatabaseConnection.databaseSingleton;
	}
	
	/**
	 * Generate the filename of the MapDB database sitting in $TMP
	 * 
	 * @return
	 */
	private static String buildDatabaseTempFileName() {
		return System.getProperty("java.io.tmpdir") + "shop.mapsdb";
	}
	
}
