package at.gv.landooe.shop.elems;

import java.util.ArrayList;
import java.util.List;

import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

/**
 * Represents a customers order
 * 
 * @author matthias
 */
@XmlRootElement
public class Order extends GenericIntIdItem {

	private static final long serialVersionUID = -7911379978518766530L;

	private String customerName;
	
	@XmlElement(name="items")
	private List<OrderItem> items;
	
	public Order(int id, String name) {
		this.setId(id);
		this.customerName = name;
	}

	public String getCustomerName() {
		return customerName;
	}

	public void setCustomerName(String customerName) {
		this.customerName = customerName;
	}

	public List<OrderItem> getItems() {
		if(this.items == null) {
			this.items = new ArrayList<OrderItem>();
		}
		return items;
	}

	public void setItems(List<OrderItem> items) {
		this.items = items;
	}
	
	/**
	 * Add an item to the order
	 * 
	 * @param item
	 * @param quantity
	 */
	public void addItem(Item item, int quantity) {
		this.getItems().add(new OrderItem(item, quantity));
	}
	
	/**
	 * Find an {@link OrderItem} item for a given item identified by id
	 * 
	 * @param item
	 * @return
	 */
	public OrderItem findOrderItem(int id) {
		for(OrderItem oi : this.getItems()) {
			if(oi.getItem() == id) {
				return oi;
			}
		}
		return null;
	}
	
	/**
	 * Compute sum over quantity
	 *  
	 * @return
	 */
	public int count() {
		int sum = 0;
		for(OrderItem oi : getItems()) {
			sum += oi.getQuantity();
		}
		return sum;
	}
	
}
