package at.gv.landooe.shop.rest;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Request;
import javax.ws.rs.core.UriInfo;

import org.apache.log4j.Logger;

import at.gv.landooe.shop.dao.ShopDAO;
import at.gv.landooe.shop.elems.Order;
import at.gv.landooe.shop.elems.OrderItem;
import at.gv.landooe.shop.html.BootstrapWrapper;

/**
 * Class capable of listing all orders and posting new orders
 * 
 * @author matthias
 */
@Path("/orders")
public class OrdersService {
	
	static Logger logger = Logger.getLogger(OrdersService.class);
	
	@Context
	UriInfo uriInfo;
	@Context
	Request request;
	
	@POST
	@Produces({ MediaType.APPLICATION_JSON, MediaType.APPLICATION_XML })
	@Consumes({ MediaType.APPLICATION_JSON, MediaType.APPLICATION_XML })
	public Order postOrder(Map<String, Object> jsonMap) {
		Order order = parseOrder(jsonMap);
		ShopDAO.saveOrder(order);
		return order;
	}
	
	/**
	 * Convert map to JSON
	 * 
	 * @param jsonMap
	 * @return
	 */
	private Order parseOrder(Map<String, Object> jsonMap) {
		Order order = new Order(-1, (String) jsonMap.get("customerName"));
		@SuppressWarnings("unchecked")
		List<Map<String, Object>> items = (List<Map<String, Object>>) jsonMap.get("items");
		List<OrderItem> orderItems = new ArrayList<>();
		for(Map<String, Object> jItem : items) {
			OrderItem oi = new OrderItem((Long) jItem.get("item"), (Long) jItem.get("quantity"), 0.0);
			orderItems.add(oi);
		}
		order.setItems(orderItems);
		return order;
	}
	
	@GET
	@Produces({ MediaType.APPLICATION_JSON, MediaType.APPLICATION_XML })
	public List<Order> getOrders() {
		return ShopDAO.listOrders();
	}
	
	@GET
	@Produces({ MediaType.TEXT_HTML })
	public String getOrdersHtml() {
		StringBuilder sb = new StringBuilder();
		sb.append("<table class=\"table\">");
		sb.append("<tr><th>");
		sb.append("Customer");
		sb.append("</th><th>");
		sb.append("Ordered Items");
		sb.append("</th><th>");
		sb.append("Details");
		sb.append("</th></tr>");
		for(Order o : ShopDAO.listOrders()) {
			sb.append("<tr><td>");
			sb.append(o.getCustomerName());
			sb.append("</td><td>");
			sb.append(o.count());
			sb.append("</td><td>");
			String fullUri = this.uriInfo.getAbsolutePath().toString() + '/' + o.getId();
			sb.append("<a href=\"");
			sb.append(fullUri);
			sb.append("\">");
			sb.append(fullUri);
			sb.append("</a>");
			sb.append("</td></tr>");
		}
		sb.append("</table>");
		return BootstrapWrapper.wrap("Open Orders", sb.toString());
	}
	
	@Path("{order}")
	public OrderService getOrder(@PathParam("order") String id) {
		return new OrderService(this.uriInfo, this.request, id);
	}
	
}
